//-----------------------------------------------------------------------
//
//	Project TibThob: a tool to search the Rangjung Yeshe Dictionary
//
//
//	This program is subject to the GNU GENERAL PUBLIC LICENSE
//	It is provided as is - WITHOUT ANY WARRANTY - in the hope
//	it will be of benefit to all beings, or at least some of them.
//	
//-----------------------------------------------------------------------
//	Copyright Xavier C. FRANC 1999
//	report bugs and comments at:
//		xfranc@dial.oleane.com , xfra@balise.com
//
//-----------------------------------------------------------------------


#include "tibbasic.h"

struct	StringList;
class	Wylie2Phonetics;
class	WylieReducer;
class	HashTable;

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//
//	class TibSearchEngine :
//

class	TibSearchEngine
{
// .......... public methods ..................................
public:
	TibSearchEngine();
	~TibSearchEngine();

	// open on a dictionary file
	TStatus		open (char *path);

	// reset options, require/exclude
	int		reset();

	// search Wylie string, specify in which areas
	// returns the number of hits
	enum Restrict { Anywhere, AtStart, InEntry, InDefinition };
	int		search (char *string, Restrict restrict);

	// search string (not yet converted), specify in which areas
	int		searchPhonetics (char *string, Restrict restrict);
	// initialise with wylie to phonetics table
	TStatus		initPhonetics (const char *tablePath);

	// search in approximate Wylie (prefix, superscript and 2nd suffix ignored)
	int		searchApproxWylie (char *string, Restrict mode);
	TStatus		initApproxWylie (const char *tablePath);

	// add a require / exclude
	void		require (char *string, Restrict restrict = Anywhere);
	void		exclude (char *string, Restrict restrict = Anywhere);
	
	// start at first hist
	void		firstHit ();
	// get next hit: return strings that must not be destroyed
	// returns 0 if no more hits
	int		getNextHit (const char * &entry, const char * &definition);
	// get next hit with conversion table (for phonetic & approxWyilie) :
	// returns the pos and size of first hit in entry. Returns 0 if no more hits
	int		getNextConvHit (const char * &entry, int &hitpos, int &hitsize,
					const char * &definition);
	// limit the number of hits (none by default)
	void		setHitMax(int max);

	// returns the current searched string
	const char	*searched ()		{  return searched_;  }

	// returns the last type of search (for display)
	int		lastSearch ()		{  return lastSearch_;  }

	// ----------- internals -----------------
protected:
	void		restart();	// before search: clear hits
	// also checks require/exclude:
	int		realHit(const char *hit, const char *end, Restrict mode, int hit);
	void		clearList (StringList *&);
	void		replace (char *&S, char *newS)	{ free(S); S = newS; }

	// search the phonetic string starting from this position
	int		phoneticMatch (const char *start, const char * &sep);
	// converts string to phonetics according to current input table 
	//  if error at position P, returns -P-1
	int		phoneticString(const char *);
	// converts to phonetics, returns end position
	int		phonConvert (const char *start, const char * &sep,
				     char phonemes[]);
	// convert an input string to phonetics according to current table
	int		input2Phonetics (const char *input, char phonemes[]);

	// search the approx wylie string starting from this position
	int		approxWylieMatch (const char *start, const char * &sep);
	// converts string to approx wylie: if error at position P, returns -P-1
	int		approxWylie(const char *);
	// converts to approx wylie, returns end position
	int		approxWylieConvert (const char *start, const char * &sep,
					    char equiv[]);

// .......... instance variables ..............................
protected:
	// search buffer (the whole file mapped in memory)
	const char	*dict_;
	int		dictSize_;
	// system data for closing file mapping
	long		mapping_;

	// searched string
	char		*searched_;
	int		searchedLen_;
	// code for last search: 'W' -> Wylie, 'A' -> approx Wylie, 'P' phonetics 
	int		lastSearch_;

	// Wylie parser and phonetics converter
	Wylie2Phonetics *w2p_;
	// approximate Wylie 
	WylieReducer	*wapp_;

	// require/exclude lists:
	StringList	*requires_;
	StringList	*excludes_;

	// hits:
	StringList	*hitEnts_;
	StringList	*hitDefs_;
	int		hitMax_;

	StringList	*curHitEnt_;
	StringList	*curHitDef_;

}; // end of class TibSearchEngine


struct	StringList
{
	StringList()
		{ chars_ = NULL; next_ = NULL; }
	~StringList()
		{ free(chars_); }

	char	   *chars_;
	StringList *next_;
};

