//-----------------------------------------------------------------------
//
//	Project TibThob: a tool to search the Rangjung Yeshe Dictionary
//
//
//	This program is subject to the GNU GENERAL PUBLIC LICENSE
//	It is provided as is - WITHOUT ANY WARRANTY - in the hope
//	it will be of benefit to all beings, or at least some of them.
//	
//-----------------------------------------------------------------------
//	Copyright Xavier C. FRANC 1999
//	report bugs and comments at:
//		xfranc@dial.oleane.com , xfra@balise.com
//
//-----------------------------------------------------------------------

#include "wyliphon.h"
#include "hashtbl.h"
#include "tibphon.h"

static	const char *const StdPhonemes[] = {
	"",
	"?",
	"K",	"KH",	"G",	"NG",
	"CH",	"CHH",	"J",	"NY",
	"T",	"TH",	"D",	"N",
	"P",	"PH",	"B",	"M",
	"TS",	"TSH",	"DZ",	"W",
	"SH",	"S",		"Y",
	"R",	"L",
	// high tone sh and s are not distinguished
	"H",
	"r",	// retroflex R
	"A",
	"E",	// dreng bu
	"I",	// gi gu
	"O",	// na ro
	"U",	// shabs kyu
	"u",	// shabs kyu
	"EU",	// shabs kyu
	"EY",	// a'i : long ey
	// other long vowels are not distinguished
	nil
};

TibPhoneme convertVowel(char v)
{
	switch(v) {
	case 'a':
	case 'A': // to improve
		return ph_A;
	case 'e':
	case 'E':
		return ph_E;
	case 'i':
	case 'I':
		return ph_I;
	case 'o':
	case 'O':
		return ph_O;
	case 'u':
	case 'U':
		return ph_U;
	default:	// should not happen
		return ph_special;
	}
}

TibPhoneme convertPhoneme (const char *symbol)
{
	for (const char * const *ph = StdPhonemes; *ph != nil; ph++)
		if (strcmp(*ph, symbol) == 0)
			return (TibPhoneme) (ph - StdPhonemes);
	fprintf(stderr, "???Pho '%s'\n", symbol);
	return ph_void;
}

// converts a string of symbols to a string of phonemes
//
#define SEPARATORS " \t\r\n"
char	*convert2Phonemes (const char *data)
{
	char phbuf[100];
	int phl = 0, pp = strspn(data, SEPARATORS);
	while (data[pp] != 0 && data[pp] != '#') {
		char symbol[20];
		int len = strcspn(data + pp, SEPARATORS);
		if (len == 0)
			break;
		memcpy(symbol, data + pp, len);
		symbol[len] = 0;
		phbuf[phl++] = convertPhoneme (symbol);
		pp += len + strspn(data + pp + len, SEPARATORS);
	}
	char *phos = new char[phl + 1];
	memcpy(phos, phbuf, phl);
	phos[phl] = ph_void;
	return phos;
}

void	convertPhonemes (const char *phons, char *buffer)
{
	buffer[0] = 0;
	for(const char *p = phons; *p != ph_void; p++) {
		strcat(buffer, StdPhonemes[*p]);
		//strcat(buffer, " ");
	}
}

TStatus	Wylie2Phonetics::loadTable (const char *path)
{
	FILE *input = fopen(path, "r");
	if (input == nil)
		return "cannot open";

	enum Area { None, Words, LeftRules, RightRules, Input };
	const int LINESIZE = 199;
	char	line[LINESIZE+1], word[100];
	Area area = None;

	for( ; fgets(line, LINESIZE, input); ) {
		if (line[0] == '#')
			continue;
		if (line[0] == '%') {
			if (strncmp(line, "%left", 5) == 0)
				area = LeftRules;
			else if (strncmp(line, "%right", 6) == 0)
				area = RightRules;
			else if (strncmp(line, "%words", 6) == 0)
				area = Words;
			else if (strncmp(line, "%input", 6) == 0)
				area = Input;
			else return "bad area";
			continue;
		}
		// an entry
		size_t L = strcspn(line, "=\t\r\n");	// blank is significant
		memcpy(word, line, L); word[L] = 0;
		size_t S = L + strspn(line + L, "= \t\r\n");
		switch(area) {
		case LeftRules:
			leftAssoc(word, convert2Phonemes(line + S));
			break;
		case RightRules:
			rightAssoc(word, convert2Phonemes(line + S));
			break;
		case Input:
			//printf("IN '%s'\n", word);
			inputTbl_->insert(convert2Phonemes(line + S), word);
			break;
		case Words:
			fprintf(stderr, "word %s -> %s\n", word, line + S);
			break;
		default:
			fprintf(stderr, "what? %s -> %s\n", word, line + S);
			break;
		}
	}
	fclose(input);
	return nil;
}

void	*Wylie2Phonetics::getRight (char *buffer)
{
	if (buffer != nil) {
		memcpy(buffer, wylie_ + leftSize_, rightSize_ + 1);
		buffer[rightSize_ + 1] = 0;
	}
	int H = getVowel() << rightSize_;
	void *r = rightTable_->find(rightHash_ + H, wylie_ + leftSize_, rightSize_ + 1);
	if (r != nil)
		return r;
	char right[40];
	right[0] = '?';
	memcpy(right + 1, wylie_ + leftSize_ + 1, rightSize_);
	right[rightSize_+1] = 0;
	H = '?' << rightSize_;
	return rightTable_->find(rightHash_ + H, right, rightSize_ + 1);
}

Wylie2Phonetics::Wylie2Phonetics()
{
	inputTbl_ = new HashTable(16);
}

Wylie2Phonetics::~Wylie2Phonetics()
{
	delete inputTbl_;
}

//	the real stuff:	
//
int	Wylie2Phonetics::convert (char phonetics[])
{
	char *left = (char*) getLeft();
	char *right = (char*) getRight();
	if (left == nil || right == nil)
		return -1;
	int L = strlen(left);
	char vowel = wylie_[leftSize_];
	if (left[0] == ph_special) {
		// special cases: db... or ba/bo
		if (wylie_[0] == 'd')
		    if (vowel == 'a')
			 phonetics[0] = ph_W;
		    else L = 0;
		else if (wylie_[0] == 'b') {
			// ba ba'i bo bo'i : brutally converted to W...
			// (this is the most frequent case)
		    if((vowel == 'a' || vowel == 'o') && 
		       (rightSize_ == 0 || wylie_[leftSize_+1] == '\''))
			 phonetics[0] = ph_W;
		    else phonetics[0] = ph_B;
		}
		else;	// should not happen
	}
	else	memcpy(phonetics, left, L);
	strcpy(phonetics + L, right);
	if (right[0] == ph_special)
		phonetics[L] = convertVowel(vowel);
	L += strlen(right);
	// euphony : nasalisation
	if (rightSize_ == 0 || rightSize_ == 1 && wylie_[leftSize_+1] == '\'') {
		const char *p = wylie_ + leftSize_ + 1 + rightSize_;
		while(*p == ' ') p++;
		if (*p == '\'') {
			phonetics[L++] = ph_N;
			phonetics[L] = 0;
		}
	}
	return L;
}


int	Wylie2Phonetics::input2Phonemes (const char *input, char phonemes[])
{
	int	php = 0;
	
	for (const char *p = input, *end = p + strlen(p); p < end; p ++)
		if (isspace(*p))
			continue;
		else {
			int L = 4;
			void *eq = inputTbl_->find(p, L);
			while (eq == nil && L > 1)
				L--, eq = inputTbl_->find(p, L);
			if (eq == nil)
				return - (p - input) - 1;
			strcpy(phonemes + php, (char*) eq);
			php += strlen(phonemes + php);
			p += L - 1;
		}
	return php;
}
