#!/bin/sh 
# Not sure where wish is installed, usually /usr/local/bin (/usr/bin
# on linuxes),  thus will not try to use ``#!/usr/local/bin/wish -f'')
# Useful trick: next line is both a Tcl comment and a shell command \
exec wish -f $0 $@ 

### ==========================================================================
###	
###	TibThob: a tool to search a Tibetan Dictionary
###
###	This program is subject to the GNU GENERAL PUBLIC LICENSE
###	It is provided as is - WITHOUT ANY WARRANTY - in the hope
###	it will be of benefit to all beings, [or at least some of them].
###	
### -----------------------------------------------------------------------
###	Copyright Xavier C. FRANC - all rights reserved
###	report bugs and comments at:
###		xfranc@dial.oleane.com , xfra@balise.com
###
### ==========================================================================
###	Note: This program is intended for the RangJung Yeshe Dictionary
###	but can work on any dictionary with the same structure
###

wm withdraw .
if {$tk_version < 8.0} {
    puts stderr \
	    "This code requires at least Tk8.0, this is only Tk$tk_version."
    exit 1
}

global tk_strictMotif
set tk_strictMotif 1

proc appliPath {filename} {
    global env

    set dir [file dirname $filename]
    if {$dir == "./."} { return [pwd] }
    if {$dir != "."} {
	return [file join [pwd] $dir]
    }
    set dirlist [split $env(PATH) ":"]
    foreach dir $dirlist {
	set pathname [file join $dir $filename]
	if {[file exists $pathname]} {
	    return [file join "." $dir]
	}
    }
    return [pwd]
}

## Basic initialisation for application
#
set Appli "TibThob"
set AppliPath [appliPath $argv0]
set ResourcePath [file join $AppliPath lib]
if { $tcl_platform(platform) == "unix" } {
    set INITPath ~/.tibthob11
} else {
    set INITPath $ResourcePath/tibthob11.ini
}

## load the extension DLL (search engine in C++)
global et_version
# different code according to packaging (normal-with-scripts vs standalone)
if { [catch { set x $et_version }] } {
    # normal mode
    set DLL [info sharedlibextension]
    load $ResourcePath/tibthob$DLL
    source $ResourcePath/helper.tcl
} else {
    # standalone executable: we dont load a DLL, it's already embedded

    source lib/helper.tcl
}


### ==========================================================================
###
###  Definition and implementation of "class" TibThobView : 
###

###------- Options and Default values ----------------------------------------

set defaultFileName "*no name*"

set OptionCat {
    { DictPath "" }
    { WylieFont Helvetica }
    { WylieFontSize 9 }
    { WylieFontStyle bold }
    { DefinFont Helvetica }
    { DefinFontSize 9 }
    { DefinFontStyle normal }
    { DefinMargin 40 }
    { HighColor blue2 }
    { Highlight 1 }
    { HitMax 200 }
    { PhonTable "" }
    { LastSearchType "" }
    { LastSearchArea "" }
    { LastGeometry "" }
}

###------- Messages and Menu data :  ----------

set InputDescriptor {
    { "simple text:" "search" }
    { "approximate syllables:" "approxsearch" }
    { "phonetics:" "phonsearch" }
}

set AreaDescriptor {
    { "in entry header" "E" }
    { "at start of entry" "S" }
    { "in definition" "D" }
    { "anywhere" "A" }
}

set RqExDescriptor {
    { "require:" "require" }
    { "exclude:" "exclude" }
}

# require/exclude area: same as for searched string, but with 'anywhere' by default
set RXAreaDescriptor {
    { "anywhere" "A" }
    { "in entry header" "E" }
    { "at start of entry" "S" }
    { "in definition" "D" }
}

###------- Menu bar:

proc createMenuBar { view }  {
    global Appli Options

    proc createMenuBarMenu { bar id title } {
	set m $bar.$id.menu		
	menubutton $bar.$id -text $title -menu $m
	menu $m
	pack $bar.$id -side left
	return $m
    }

    frame $view.mbar -relief groove -bd 1
    pack $view.mbar -side top -fill x

    set m [createMenuBarMenu $view.mbar files "Files"]
        $m add command -label "New View" -command "fileCmdAddView" -accelerator Ctrl-N
        $m add command -label "Export to text file..." -command "fileCmdExport txt $view"
        #### $m add command -label "Export to RTF..." -command "fileCmdExport rtf $view"
        $m add separator
        $m add command -label "Exit" -command "fileCmdExit $view" -accelerator Ctrl-Q

    set m [createMenuBarMenu $view.mbar view "Options"]
        $m add command -label "Setup Dictionary" -command "fileCmdOpen $view"
        $m add command -label "Display" -command "cmdSettings $view"
        $m add command -label "Phonetics" -command "cmdPhonSettings $view"

    set m [createMenuBarMenu $view.mbar help "Help"]
        $m add command -label "About $Appli" -command "helpAbout $view"
        $m add command -label "Manual" -command "helpIntro $view"
        #$m add command -label "Tutorial"	-command "helpTutor $view"

    pack $view.mbar.help -side right -ipadx 10
}

####### TibThobView ##################################################
 #	
 #	Creation and management of an edition view
 #	 usage: TibThobView windowId ?file
 #	
##
proc TibThobView { box {inputFile ""} } {

    ## Creation and initialisation
    proc tibBoxCreate {box} {

	global Appli
	set wclass $Appli

	### Window manager:
	toplevel $box -class $wclass
	wm withdraw $box
	wm protocol $box WM_DELETE_WINDOW  "viewDestroy $box"
	wm resizable $box yes yes

	### Widget default resources:
	option add *$wclass.geometry +10+10 \
		widgetDefault
	# size of results area:
	option add *$wclass.results.sc.text.height 24 \
		widgetDefault
	option add *$wclass.results.sc.text.width 88 \
		widgetDefault
	option add *$wclass*Entry.background White \
		widgetDefault

	#----- menu bar :

	createMenuBar $box

	### ----- command area :
	frame $box.commands -relief groove 
	pack $box.commands -fill x -padx 6 -pady 6

	## 'Search' button
	button $box.commands.go -text " SEARCH " -command "cmdSearch $box"
	pack $box.commands.go -side left -padx 10

	## search area
	global AreaDescriptor 
	makeOptionMenu $box.commands.mode $AreaDescriptor SearchMode($box) AreaCode
	pack $box.commands.mode -side left

	## mode selector:
	global InputDescriptor InputKind InputCode 
	makeOptionMenu $box.commands.inKind $InputDescriptor InputKind($box) InputCode
	#$box.commands.inKind configure -indicatoron 0
	pack $box.commands.inKind -side left

	## searched text input:
	set inp $box.commands.input
	entry $box.commands.input -width 15
	pack $box.commands.input -side left -fill both -expand yes

	bind $inp <Return> "cmdSearch $box"
	# this clears the input before <Button-2> inserts the selection:
	bind $inp <Control-ButtonPress-2> { %W delete 0 end }
	bind $inp <3> { tkEntryPaste %W %x }
	bind $inp <Control-ButtonPress-3> { %W delete 0 end ; tkEntryPaste %W %x}

	#bind $inp <Page_Up> "[resultsForm $box] yview scroll -1 pages"
	#bind $inp <Page_Down> "[resultsForm $box] yview scroll 1 pages"
	bind $inp <Up> "[resultsForm $box] yview scroll -1 units "
	bind $inp <Down> "[resultsForm $box] yview scroll 1 units"

	bind $inp <Control-n> fileCmdAddView
	bind $inp <Control-q> "fileCmdExit $box"

	bind $inp <Alt-a> \
	    "set SearchMode($box) \"[comboByCode $AreaDescriptor A]\""
	bind $inp <Alt-d> \
	    "set SearchMode($box) \"[comboByCode $AreaDescriptor D]\""
	bind $inp <Alt-e> \
	    "set SearchMode($box) \"[comboByCode $AreaDescriptor E]\""
	bind $inp <Alt-s> \
	    "set SearchMode($box) \"[comboByCode $AreaDescriptor S]\""

	bind $inp <Alt-t> \
	    "set InputKind($box) \"[comboByCode $InputDescriptor search]\""
	bind $inp <Alt-x> \
	    "set InputKind($box) \"[comboByCode $InputDescriptor approxsearch]\""
	bind $inp <Alt-p> \
	    "set InputKind($box) \"[comboByCode $InputDescriptor phonsearch]\""

	 # set the keyboard focus to this input area:
	focus $box.commands.input

	# uncomment to add a "Clear" button at right of input field:
	##button $box.commands.clear -text "Clr" -command "$inp delete 0 end"
	##pack $box.commands.clear -side right

	### ----- refining search: create only the 'Refine' button
	#         but enclose it in a box that will contain the require/exclude lines

	frame $box.refines -relief groove 
	pack $box.refines -fill x -padx 16 -pady 2
	frame $box.refines.c -relief groove 
	pack $box.refines.c -side top -fill x -expand yes
	button $box.refines.c.b -command "cmdRefine $box" -text " Refine... "
	pack $box.refines.c.b -side left
	global Refines
	set Refines($box) 1

	### ----- scrolled canvas containing the results:

	frame $box.results -relief sunken -borderwidth 2
	label $box.results.label -text "Results:" -justify left 
	pack $box.results.label -side top 
	frame $box.results.sc
	yscrolledText $box.results.sc [resultsForm $box]
	pack $box.results.sc -side bottom -fill both -expand yes
	pack $box.results -fill both -expand yes

	setStyles [resultsForm $box]

	tiblClearView $box
    }

  ##---- Command callbacks --------------------------------------

    proc fileCmdExit {view} {
	saveViewSettings $view
	allDone
    }
    
    proc fileCmdOpen {view} {
	set cur [getOpt DictPath]
	set dir [file dir $cur]
	if { ! [file exists $dir ] } {
	    set dir [pwd]
	}
	set title "Setup Tibetan Dictionary"

	set fileName [tk_getOpenFile -title $title -initialdir $dir -initialfile $cur ]
	if { $fileName != "" } {
	    setOpt DictPath $fileName
	    tibdix dictionary $fileName
	}
    }
    
    proc fileCmdExport {mode view} {
	set file [tk_getSaveFile -initialdir [pwd] -title "Export" \
		-defaultextension $mode -initialfile "[string trim $view .].$mode" ]
	if { $file == "" } return
	tibdix export $mode $file
    }

    proc fileCmdAddView {} {
	set i 0
	while 1 {
	    incr i
	    if [winfo exists .view$i] { continue }
	    TibThobView .view$i
	    break
	}
    }

    proc cmdSearch {view}   { 
	global InputKind InputCode SearchMode AreaCode
	doSearch $InputCode($InputKind($view)) $view $AreaCode($SearchMode($view)) \
		[$view.commands.input get]
    }

    proc cmdRefine {view} {
	global Refines RefineType RqExDescriptor RefineCode 
	global RefineText RefineArea RXAreaDescriptor AreaCode 

	set cnt $Refines($view) 
	incr Refines($view)
	set box $view.refines.$cnt
	frame $box -relief groove 

	# require/exclude
	makeOptionMenu $box.type $RqExDescriptor RefineType($box) RefineCode 
	#$box.type configure -indicatoron 0

	# input area (Wylie only)
	entry $box.text -width 30
	bind $box.text <Return> "cmdSearch $view"

	# select area
	makeOptionMenu $box.area $RXAreaDescriptor RefineArea($box) AreaCode 

	# 'clear' and 'dismiss' buttons:
	button $box.clear -text " clear " -command "$box.text delete 0 end"
	button $box.kill -text " dismiss " -command "cmdDerefine $view $box"

	pack $box.type $box.text $box.area -side left 
	pack $box.kill $box.clear -side right
	pack $box -fill x

	$view.refines.c.b configure -text "Refine more..."
    }

    # kill a refine line:
    proc cmdDerefine {view refine} {
	destroy $refine
	if { [llength [winfo children $view.refines]] <= 1 } {
	    $view.refines.c.b configure -text "Refine..."
	}
    }

    proc viewDestroy { view } {
	saveViewSettings $view
	destroy $view
	global ViewCount
	incr ViewCount -1
	if { $ViewCount == 0 } {
	    allDone
	}
    }

    proc helpAbout { view } {
	global Appli
	if { ![winfo exists .about] } {
	    set w .about
	    option add $w.wrapLength 7i 50
	    GeneralDialog $w "About $Appli"
	    $w.ok config -text "'grigs ?" -justify center
	    destroy $w.cancel 

	    text $w.work.text -relief groove
	    $w.work.text configure -wrap word -height 29
	    $w.work.text tag configure big -font { helvetica 12 bold } -justify center
	    $w.work.text insert end "\n$Appli 1.1\n" big
	    $w.work.text tag configure small -font { helvetica 10 } -justify center
	    $w.work.text insert end "
This program is free software. It is provided as is - without any warranty - in the hope it will benefit all beings, \n
\[or at least those who are interested in Tibetan language]\n
CopyrightXavierC.FRANC1999-2002\nxfranc@online.fr\n\n" small
	    $w.work.text tag configure ita -font { helvetica 10 italic } -justify center -spacing3 -5
	    $w.work.text insert end "
Bsod nams 'di yis thams chad gzigs pa nyid\n
thob nas nyes pa'i dgra rnams pham byas nas\n
skye rga na 'chi'i rba rlabs 'khrugs pa yi\n
srid pa'i mtsho las 'gro ba sgrol bar shog" ita
            $w.work.text configure -state disabled
	    pack $w.work.text -expand yes -fill both
	}
	dialogPlace .about $view
    }

    proc helpIntro { view } {
	helpWindow .help $view
	global ResourcePath Appli
	helpWindowOpen .help $ResourcePath/hintro.thl "$Appli Help: user manual" 
    }

    proc helpTutor { view } {
	helpWindow .help $view
	global ResourcePath Appli
	helpWindowOpen .help $ResourcePath/htutor.thl "$Appli Help: Tutorial" 
    }

 ### -------- Utility functions :

    proc resultsForm {view} { return $view.results.sc.text }

    ##  Reset the contents of a view :
    proc tiblClearView {view} {
	global TiblEditObj Appli

	set TiblEditObj(saveFile$view) ""
	set TiblEditObj(inputFile$view) ""
	# command area:
	set TiblEditObj(Command$view) ""
	[resultsForm $view] delete 0.0 end
	wm title $view "$Appli: Tibetan Dictionary Search"
    }

    proc doSearch { searchType view mode what } {

	if { [string length $what] == 0 } return
	global DictPath RefineArea AreaCode RefineType RefineCode
	# prepare search:
	set err ""
	
	if { [catch { tibdix reset } err] } {
	    if { "$err" == "dictionary non initialised" } {
		set err "Please setup the Tibetan dictionary\nusing the Options / Dictionary menu"
	    }
	    tellError $err
	    return
	}
	set max [getOpt HitMax]
	tibdix hitmax $max
	#  send requires and excludes:
	foreach refw [winfo children $view.refines ] {
	    if { $refw == "$view.refines.c" } continue
	    set rx [$refw.text get] 
	    if  { [string trim $rx " \t\r"] == "" } continue
	    tibdix $RefineCode($RefineType($refw)) $rx $AreaCode($RefineArea($refw))
	}
	# set cursor as a watch:
	set savecu [$view cget -cursor]
	$view configure -cursor watch
	update
	# actual search:
	set hits [tibdix $searchType $what $mode]
	# display 
	redisplayHits $view

	set fmt  "Results: %d items"
	if { $hits >= $max } {
	    set fmt "$fmt (%d displayed)"
	    [resultsForm $view] insert end "\n\[ and more ... ]\n\n" WylieHigh  
	} elseif { $hits < 0 } {
	    set fmt  "improper input !"
	    $view.commands.input selection range [expr -1 - $hits ] end
	}
	$view.results.label configure -text [format $fmt $hits $max]

	$view configure -cursor $savecu
	update
    }

    proc redisplayHits {view} {
	global Highlight
	[resultsForm $view] delete 0.0 end
	tibdix display [resultsForm $view] [getOpt Highlight]
    }

    # initialise the styles for text display:
    proc setStyles {view} {
	$view configure -wrap word
	set ft [list [getOpt WylieFont] [getOpt WylieFontSize] [getOpt WylieFontStyle]]
	$view tag configure WylieEntry -font $ft
	$view tag configure WylieHigh -font $ft -foreground [getOpt HighColor]
	set ft [list [getOpt DefinFont] [getOpt DefinFontSize] [getOpt DefinFontStyle]]
	set mrg [getOpt DefinMargin]
	$view tag configure Definition -font $ft -lmargin1 $mrg -lmargin2 $mrg
	$view tag configure DefHigh -font $ft -lmargin1 $mrg -lmargin2 $mrg \
		-foreground [getOpt HighColor]
    }

    ## --------- Body of TibThobView -------------

    if {![winfo exists $box]} {
	tibBoxCreate $box
    }

    update idletasks
    wm deiconify $box
    update
    raise $box
    update idletasks
    update

    if { [catch { tibdix reset } err] } {
	fileCmdOpen $box
    } 

    global ViewCount
    incr ViewCount
}

##--------------------------------------------------------------------------------
 #  Text :
 #

# redefine Delete key in a more common way (like backspace)
#bind Text <Delete> {
#    if {[%W tag nextrange sel 1.0 end] != ""} {
#        %W delete sel.first sel.last
#    } elseif [%W compare insert != 1.0] {
#        %W delete insert-1c
#        %W see insert
#    }
#}

##--------------------------------------------------------------------------------
 #  Create a y scrolled frame named 'name' inside 'parent'
 #        the parent is assumed to be an existing frame
 #
proc yscrolledText {parent name} {
    scrollbar $parent.yscroll -relief sunken -command "$parent.text yview"
    pack $parent.yscroll -side right -fill y 

    text $parent.text -relief groove -width 80 -borderwidth 3 \
	    -yscrollcommand "$parent.yscroll set" 
    pack $parent.text -side left -fill both -expand yes
}

##--------------------------------------------------------------------------------
 #  Option menu: create with labels/codes descriptor
 #
proc makeOptionMenu { box descriptor primaryVar secondaryVar } {
    set labels {}
    global $secondaryVar
    foreach item $descriptor {
	set label [lindex $item 0]
	lappend labels $label
	# second item is the value stored in secondaryVar
	array set $secondaryVar [list $label [lindex $item 1]]
    }
    return [eval [concat [list tk_optionMenu $box $primaryVar ] $labels]]
}

proc comboByCode { data code } {
    foreach item $data {
	if { [lindex $item 1] == $code } {
	    return [lindex $item 0]
	}
    }
}

##--------------------------------------------------------------------------------
#	Option settings
#
#
proc cmdSettings { { view . } } {

    set D .settings.work
    if {! [winfo exists .settings]} {
	global Appli
	GeneralDialog .settings "$Appli Settings" "finishSettings"

	button .settings.buttons.apply -text "Apply" -command "applySettings .settings"
	pack .settings.buttons.apply -side left

	scaleChooser $D.hits "Displayed Hits (max)" 0 2000 HitMax
	$D.hits.scale configure -resolution 10 -length 220
	fontChooser $D.wyliefont "Wylie's Font"
	fontChooser $D.deffont "Definition's Font"
	# highlight selection:
	frame $D.high
	checkbutton $D.high.select -width 18 -text "Highlight Hits" -variable Highlight 
	colorChooser $D.high.color "Highlight color"
	pack $D.high.select $D.high.color -side left
	scaleChooser $D.margin "Definitions Left margin" 0 100 DefinMargin
	pack $D.hits $D.high $D.wyliefont $D.deffont $D.margin -padx 6 -pady 2 -fill both
    }

    global fontChooser colorChooser Highlight DefinMargin HitMax
    set ft $D.wyliefont
    set fontChooser($ft.family) [getOpt WylieFont]
    set fontChooser($ft.size) [getOpt WylieFontSize]
    set fontChooser($ft.style) [getOpt WylieFontStyle]
    set ft $D.deffont
    set fontChooser($ft.family) [getOpt DefinFont]
    set fontChooser($ft.size) [getOpt DefinFontSize]
    set fontChooser($ft.style) [getOpt DefinFontStyle]
    set colorChooser($D.high.color) [getOpt HighColor]
    $D.high.color config -foreground [getOpt HighColor]

    set Highlight [getOpt Highlight]
    set DefinMargin  [getOpt DefinMargin]
    set HitMax  [getOpt HitMax]
    dialogPlace .settings $view
}

proc finishSettings { dialog } {
    applySettings $dialog
    wm withdraw $dialog
    saveSettings
}

proc applySettings { dialog } {
    global fontChooser colorChooser Highlight DefinMargin HitMax

    set ft .settings.work.wyliefont
    setOpt WylieFont $fontChooser($ft.family)
    setOpt WylieFontSize $fontChooser($ft.size)
    setOpt WylieFontStyle $fontChooser($ft.style)
    set ft .settings.work.deffont
    setOpt DefinFont $fontChooser($ft.family)
    setOpt DefinFontSize $fontChooser($ft.size) 
    setOpt DefinFontStyle $fontChooser($ft.style)
    setOpt HighColor $colorChooser(.settings.work.high.color)
    setOpt Highlight $Highlight
    setOpt DefinMargin $DefinMargin
    setOpt HitMax $HitMax

    foreach view [winfo children .] {
	if [string match .view* $view] {
	    setStyles [resultsForm $view]
	    redisplayHits $view
	}
    }
}

proc phonHeader { path } {
    set header 0
    catch { 
	set chan [open $path r]
	set header [gets $chan]
	close $chan
	set header [join [lrange [split $header] 1 end]]
    }
    return $header
}

proc cmdPhonSettings { { view . } } {
    if {! [winfo exists .phsettings]} {
	global Appli ResourcePath PhonTable1
	set D .phsettings.work
	GeneralDialog .phsettings "$Appli Phonetic Settings" "finishPhonSettings"
	set menu {}
	foreach phontbl [lsort [glob -nocomplain $ResourcePath/*.pho]] {
	    set header [phonHeader $phontbl]
	    if { $header != 0 } {
		lappend menu [list $header $phontbl]
	    }
	}
	frame $D.tables
	label $D.tables.label -width 15 -text "Phonetic table:" -justify left 
	makeOptionMenu $D.tables.menu $menu PhonTable1 PhonTableName
	pack $D.tables.label $D.tables.menu -side left -padx 10
	pack $D.tables
    }
    set PhonTable1 [phonHeader [getOpt PhonTable]]
    dialogPlace .phsettings $view
}

proc finishPhonSettings { dialog } {
    wm withdraw $dialog
    global PhonTableName PhonTable1
    setOpt PhonTable $PhonTableName($PhonTable1)
    tibdix phonetics $PhonTableName($PhonTable1)
}

proc saveSettings { } {
    global OptionCat INITPath
    catch {
	set out [open $INITPath w]
	puts $out "# options for TibDix"
	foreach op $OptionCat {
	    set opname [lindex $op 0]
	    puts $out "setOpt $opname \"[getOpt $opname]\""
	}
	flush $out
	close $out
    }
}
	
proc saveViewSettings { view } {
    global InputKind SearchMode
    setOpt LastGeometry [winfo geometry $view]
    setOpt LastSearchType $InputKind($view)
    setOpt LastSearchArea $SearchMode($view)
}

proc allDone { } {
    saveSettings
    exit 0
}

#	set submenu [menu $m.fonts ]
#	$m add cascade -label "Tibetan Font" -state disabled -menu $submenu
#	  $submenu add radiobutton -label "none (Wylie)" -command "viewFont $view {}"
#	  $submenu add radiobutton -label "SAMW" -command "viewFont $view SAMW"
#	  $submenu add radiobutton -label "LTibetan" -command "viewFont $view LTibetan"
#	  $submenu add radiobutton -label "TFW8" -command "viewFont $view TFW8"
#	  $submenu add radiobutton -label "Uchen" -command "viewFont $view Uchen"

proc GeneralDialog { w title {callback "wm withdraw"} {parent .} } { 

    if {[winfo exists $w]} { return $w }
    toplevel $w -class Dialog
    # On windows, transient has a very strange effect: the dialog is invisible!
    #wm transient $w [winfo toplevel [winfo parent $w]]
    wm title $w $title
    #wm protocol $w WM_DELETE_WINDOW { }
    #wm resizable $w yes yes
    frame  $w.buttons -relief groove -borderwidth 1
    button $w.ok     -text "OK" -command "$callback $w" 
    button $w.cancel -text "Cancel" -command "wm withdraw $w; grab release $w"
    pack   $w.buttons -side bottom -fill x
    pack   $w.ok     -in $w.buttons -side left  -padx 3m -pady 2m
    pack   $w.cancel -in $w.buttons -side right -padx 3m -pady 2m
    frame  $w.work
    pack   $w.work -side top -expand yes -fill both
    return $w
}

##  Place a dialog in the center of the parent :
 #
proc dialogPlace { win parent } {
    # to be sure actual size is OK:
    wm withdraw $win
    update idletasks
    set pw [winfo reqwidth $parent]
    set ph [winfo reqheight $parent]
    set w [winfo reqwidth $win]
    set h [winfo reqheight $win]
    set x [expr [winfo rootx $parent] + ( $pw - $w ) / 2 ]
    set y [expr [winfo rooty $parent] + ( $ph - $h ) / 2 ]
    wm geometry $win $w\x$h+$x+$y
    raise $win
    update
    wm deiconify $win
    update
}

proc fontChooser { win label } {
    global fontChooser
    frame $win
    label $win.label -width 20 -justify right -text $label
    set family [eval [concat [list tk_optionMenu $win.family fontChooser($win.family) ] \
	    		     [font families]]]
    $win.family config -width 15
    set size [tk_optionMenu $win.size fontChooser($win.size) 8 9 10 11 12 13 14 15 16 ]
    $win.size config -width 3
    set style [tk_optionMenu $win.style fontChooser($win.style) normal bold italic]
    $win.style config -width 8
    pack $win.label $win.family $win.size $win.style -side left
}

proc scaleChooser { win label from to variable } {
    frame $win
    label $win.label -width 20 -justify right -text $label
    scale $win.scale -orient horizontal -length 150 -from $from -to $to \
	    -tickinterval $to -variable $variable
    pack $win.label $win.scale -side left
}

proc colorChooser { win label } {
    proc selectColor { win } {
	global colorChooser
	set col [tk_chooseColor -initialcolor $colorChooser($win) -parent $win]
	if { $col != "" } {
	    set colorChooser($win) $col
   	    $win config -foreground $col
	}
    }
    global colorChooser
    button $win -text $label -command "selectColor $win"
    set colorChooser($win) black
}

##--------------------------------------------------------------------------------
 #  Miscellaneous :
 #

##  Compare two strings
 #
proc streq {s1 s2} {
    return [expr [string compare $s1 $s2] == 0]
}

##  Pad a string with blanks up to a given size (if already longer, add one blank)
 #   
set Blank "                                                                      "

proc padString { string size } {
    set L [string length $string]
    global Blank
    if { $L < $size } {
	set res [string range $Blank 0 [expr $size - $L]]
	return "$string$res"
    } else {
	return "$string "
    }
    return $res
}

##  Return a temporary file name:
 #
proc fileTmpName { {seed "/tmp/xxx"} } {
    return "$seed[pid]"
}

##  Delete a file :
 #
proc fileDelete { path } {
    catch "exec /bin/rm -f $path" status
    return $status
}

##  Post an error with confirmation
 #
proc tellError {text} {
    bell
    tk_dialog .msg "Error:" "$text" error 0 OK
}

##  Display a fatal error message and exit
 #
proc tellFatal {text} {
    bell
    tk_dialog .msg "Fatal:" "$text" error 0 OK
    exit 7
}

proc setOpt { name value } {
    global Options
    set Options($name) $value
}

proc getOpt { name } {
    global Options
    return $Options($name)
}

### ==========================================================================
###    default look:

#option add *font "-*-helvetica-bold-r-normal--11-*-*-*-*-*-*-*"
option add *Dialog*wrapLength 4i 30

### ==========================================================================
### Main program 
### ==========================================================================

### Global options:
set ViewCount 0

### Initialisation, argument parsing :
###
proc appliInit {} {

    global Appli

    # initialise options:
    global OptionCat INITPath ResourcePath
    foreach op $OptionCat {
	setOpt [lindex $op 0] [lindex $op 1]
    }
    setOpt DictPath $ResourcePath/dictiona.txt

    # try to load init file:
    if { [file isfile $INITPath] } {
	# file nativename is to turn around mktclapp
	source [file nativename $INITPath]
    }
    # try to load dictionary:
    set dict [getOpt DictPath]
    if { [file isfile $dict] } {
	tibdix dictionary $dict
    }

    # try to load Wylie approximate table
    set tbl $ResourcePath/wylieapp.tbl
    if { [file isfile $tbl] } {
	tibdix approx $tbl
    } else {
	tk_dialog .msg "" "cannot load Wylie-Approximate table" warning 0 OK
    }

    # try to init phonetic table
    set pht [getOpt PhonTable]
    if { $pht == "" || ! [file isfile [getOpt PhonTable]] } {
	setOpt PhonTable $ResourcePath/peloose.pho
    }
    if { [file isfile [getOpt PhonTable]] } {
	set err ""
	catch { tibdix phonetics [getOpt PhonTable] } err
	if { $err != "" } {
	    tellError "error loading phonetic table \'[getOpt PhonTable]' : $err"
	}
    } else {
	tk_dialog .msg "" "cannot load Wylie-Approximate table" warning 0 OK
    }

    set geo [getOpt LastGeometry]
    if { $geo != "" } {
	option add $Appli*geometry $geo startupFile
    }
    global InputKind SearchMode
    if { [getOpt LastSearchType] != "" } {
	set InputKind(.view1) [getOpt LastSearchType]
	set SearchMode(.view1) [getOpt LastSearchArea]
    }

}

### Start sequence:
 appliInit
 # start with (first) view :
 TibThobView .view1
